/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.client;

import java.util.ArrayList;
import java.util.Collection;

import de.bsvrz.ibv.uda.interpreter.UdaParserFehler;

/**
 * repräsentiert das Ergebnis der lokalen Übersetzung eines UdaSkripts.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ParserResult {

	/**
	 * die Liste der bei der Übersetzung aufgetretenen Fehler.
	 */
	private final Collection<UdaParserFehler> fehlerListe = new ArrayList<>();

	/**
	 * liefert den Status der Übersetzung.
	 *
	 * @return <i>true</i>, wenn die Übersetzing erfolgreich war
	 */
	public final boolean isParsed() {
		return fehlerListe.isEmpty();
	}

	/**
	 * liefert die Liste aller aufgetretenen Fehler.
	 *
	 * @return die Liste
	 */
	public Collection<UdaParserFehler> getFehlerListe() {
		return fehlerListe;
	}

	/**
	 * fügt der Fehlerliste ein Fehlerobjekt hinzu.
	 *
	 * @param fehler
	 *            der hinzuzufügende Fehler
	 */
	public void addFehler(final UdaParserFehler fehler) {
		fehlerListe.add(fehler);
	}
}
