/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.client;

import de.bsvrz.ibv.uda.uda.data.Modul;
import de.bsvrz.ibv.uda.uda.data.Skript;
import de.bsvrz.ibv.uda.uda.data.SkriptLauf;

/**
 * Allgemeine Datenrepräsentation eines Skriptlaufes.<br>
 * Die Daten müssen keine Zuordnung zu einem entsprecheden Objekt innerhalb der
 * Datenverteilerkonfiuguration besitzen. Instanzen dieser Klasse werden zum
 * einen zum temporären Zwischenspeichern von Informationen zu einem Skriptlauf
 * bzw. zum Ausführen von lokalen Skripten ohne Einbeziehung der "Umfassenden
 * Datenanalyse"-Anwendung eingesetzt.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class SkriptLaufDaten extends SkriptLauf {

	/**
	 * das Modul, dem der Skriptlauf zugeordnet ist.
	 */
	private Modul modul;

	/**
	 * Konstruktor.<br>
	 * Die Funktion erzeugt ein Objekt der Klasse. Das Modul muss nicht zwingend
	 * angegeben werden, d.h. es kann der Wert <b><i>null</i></b> übergeben
	 * werden.
	 *
	 * @param modul
	 *            das Modul, dem die Laufdaten zugeordnet sind
	 * @param name
	 *            der Name des Skriptlaufs
	 * @param skript
	 *            das auszuführende Skript
	 */
	public SkriptLaufDaten(final Modul modul, final String name, final Skript skript) {
		this.modul = modul;
		setName(name);
		setSkript(skript);
	}

	/**
	 * Konstruktor.<br>
	 * Die Funktion erzeugt ein SkriptDatenObjekt mit den Daten des übergebenen
	 * Skriptlaufs.
	 *
	 * @param lauf
	 *            der ursprüngliche Skriptlauf
	 */
	public SkriptLaufDaten(final SkriptLauf lauf) {
		kopiereDaten(lauf);
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.uda.data.SkriptLauf#getModul()
	 */
	@Override
	public Modul getModul() {
		return modul;
	}

	/**
	 * {@inheritDoc}.
	 *
	 * @see de.bsvrz.ibv.uda.uda.data.SkriptLauf#publiziereErgebnis(java.lang.Exception)
	 */
	@Override
	public void publiziereErgebnis(final Exception fehler) {
		// nichts zu tun
	}
}
