/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Timer;
import java.util.WeakHashMap;

import de.bsvrz.dav.daf.main.ApplicationCloseActionHandler;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.iav.fuzzylib.interpreter.logik.FuzzyLogikHandler;
import de.bsvrz.ibv.uda.interpreter.anweisung.UdaAktivitaet;
import de.bsvrz.ibv.uda.interpreter.anweisung.anmeldung.AnmeldungHandler;
import de.bsvrz.ibv.uda.interpreter.ausdruck.FunktionsDefinition;
import de.bsvrz.ibv.uda.interpreter.ausdruck.QuantorDefinition;
import de.bsvrz.ibv.uda.interpreter.daten.ArithmetikHandler;
import de.bsvrz.ibv.uda.interpreter.daten.KonfigurationsKontext;
import de.bsvrz.ibv.uda.interpreter.daten.KonstantSkriptKontext;
import de.bsvrz.ibv.uda.interpreter.daten.ParameterSkriptKontext;
import de.bsvrz.ibv.uda.interpreter.daten.SkriptKontext;
import de.bsvrz.ibv.uda.interpreter.daten.ThreadHandler;
import de.bsvrz.ibv.uda.interpreter.daten.ZeichenkettenHandler;
import de.bsvrz.ibv.uda.interpreter.daten.container.ContainerHandler;
import de.bsvrz.ibv.uda.interpreter.daten.dav.DavHandler;
import de.bsvrz.ibv.uda.interpreter.daten.dav.KonfigurationsHandler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.FehlerHandler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.UdaFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fuzzy.UdaFuzzyHandler;
import de.bsvrz.ibv.uda.interpreter.daten.hysterese.HystereseHandler;
import de.bsvrz.ibv.uda.interpreter.daten.java.JavaHandler;
import de.bsvrz.ibv.uda.interpreter.daten.struktur.DatenStrukturHandler;
import de.bsvrz.ibv.uda.interpreter.daten.trend.TrendHandler;
import de.bsvrz.ibv.uda.interpreter.daten.zeit.ZeitstempelHandler;
import de.bsvrz.ibv.uda.uda.UdaSkriptAusfuehren;
import de.bsvrz.ibv.uda.uda.data.SkriptLauf;
import de.bsvrz.ibv.uda.verwaltung.ServerSkriptLauf;
import de.bsvrz.ibv.uda.verwaltung.protokoll.ProtokollEintrag;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Ausdruck;
import de.bsvrz.sys.funclib.bitctrl.interpreter.InterpreterException;
import de.bsvrz.sys.funclib.bitctrl.interpreter.NichtWert;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;

/**
 * die Repräsentation eines Uda-Interpreter.<br>
 * Ein Uda-Interpreter wird als einzelner Thread gestartet und führt ein
 * SkriptLauf-Objekt aus, d.h. ein Skript mit einer zugewiesenen
 * Ausführungsumgebung.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class UdaInterpreter extends Thread implements ApplicationCloseActionHandler {

	/**
	 * Marker, für die Registrierung der Handler für die Ausführung von
	 * Operationen.<br>
	 * Alle Handler werden nur einmalig global registriert.
	 */
	private static boolean handlerRegistriert;

	/**
	 * prüft, ob Daten einem Zielobjekt zugewiesen werden können und nimmt
	 * gegebenenfalls eine Konvertierung vor.
	 *
	 * @param dest
	 *            die alten Daten des Ziels
	 * @param daten
	 *            die Daten, die zugewiesen werden sollen
	 * @return die zuzuweisenden Daten
	 */
	public static Object getZuweisbareDaten(final Object dest, final Object daten) {
		Object result = null;

		if (dest == null) {
			result = daten;
		} else if (dest.getClass().isAssignableFrom(daten.getClass())) {
			result = daten;
		} else if (dest.equals(NichtWert.NICHTWERT)) {
			result = daten;
		} else if ((dest instanceof Number) && (daten instanceof Number)) {
			result = daten;
		} else if (dest instanceof String) {
			result = daten.toString();
		} else if (daten.equals(NichtWert.NICHTWERT)) {
			result = daten;
		} else if (daten instanceof UdaFehler) {
			result = daten;
		}

		return result;
	}

	/**
	 * führt den Interpreter lokal aus.
	 *
	 * @param args
	 *            die Kommandozeilen-Parameter
	 */
	public static void main(final String[] args) {
		StandardApplicationRunner.run(new UdaSkriptAusfuehren(), args);
	}

	/**
	 * der Skriptlauf, der mit dem Interpreter ausgeführt wird.
	 */
	private final SkriptLauf skriptLauf;

	/**
	 * der Kontext für die Ausführung des Skripts.
	 */
	private final SkriptKontext kontext = new SkriptKontext(this);

	/**
	 * die Liste der nebenläufigen Aktivitäten, die neben dem Hauptskript
	 * ausgeführt werden.
	 */
	private final WeakHashMap<UdaAktivitaet, UdaAktivitaet> aktivitaeten = new WeakHashMap<>();

	/**
	 * der Anweisungsblock des Hauptskripts.
	 */
	private final AnweisungsBlock anweisungsBlock = new AnweisungsBlock(0, "");

	/**
	 * die Datenverteilerverbindung, über die das Skript kommuniziert.
	 */
	private ClientDavConnection connection;

	/**
	 * der Timer zur Ausführung von zyklischen Operationen und
	 * Timeout-Überwachungen.
	 */
	private final Timer skriptTimer = new Timer();

	/**
	 * die Liste aller nutzerdefinierten Funktionen.
	 */
	private final Map<String, Object> funktionen = new HashMap<>();

	/**
	 * die Liste aller nutzerdefinierten Quantoren.
	 */
	private final Map<String, Object> quantoren = new HashMap<>();

	/**
	 * Menge der Interpreterlistener.<br>
	 * Ein Interpreterlistener wird benachrichtigt, wenn der Interpreter die
	 * Abarbeitung abgeschlossen oder auf andere Weise beendet hat.
	 */
	private final Set<InterpreterListener> listenerListe = new HashSet<>();

	/**
	 * die initiale Verbindung des Interpreters, d.h. die Verbindung mit dem die
	 * umgebende Anwendung ausgeführt wird. Der Interpreter verwendet die
	 * gleichen Verbindungsparameter.
	 */
	private final ClientDavInterface initialConnection;

	/**
	 * markiert den Laufzustand des Interpreters.
	 */
	private boolean running;

	/**
	 * die Fehlermeldung beim Abbruch der Datenverteilerverbindung des
	 * Interpreters.
	 */
	private String connectionError;

	/**
	 * Konstruktor.<br>
	 * Die Funktion erstellt einen Interpreter aus dem übergebenen
	 * Skriptlaufobjekt. Der zugeordnete Quelltext des Skripts wird analysiert
	 * und eine für den Interpreter ausführbare Form übersetzt.
	 *
	 * @param initialConnection
	 *            die Verbindung der umgebenden Anwendung
	 * @param skriptLauf
	 *            den SkriptLauf
	 * @throws UdaInterpreterException
	 *             bei der Erstellung des Interpreters ist ein Fehler
	 *             aufgetreten. Mögliche Probleme sind:
	 *             <ul>
	 *             <li>die Datei mit dem Quelltext konnte nicht ermittelt
	 *             werden,</li>
	 *             <li>beim Einlesen der Quelldatei wurden ungültige Zeichen
	 *             empfangen,</li>
	 *             <li>der Quelltext enthält syntaktische Fehler.</li>
	 *             </ul>
	 */
	public UdaInterpreter(final ClientDavInterface initialConnection, final SkriptLauf skriptLauf)
			throws UdaInterpreterException {
		super(skriptLauf.getName());
		this.skriptLauf = skriptLauf;
		this.skriptLauf.setInterpreter(this);
		this.initialConnection = initialConnection;

		try {
			InterpreterFactory.createInterpreterBaum(true, this, skriptLauf.getSkript());
		} catch (final UdaInterpreterException e) {
			skriptLauf.publiziereErgebnis(e);
			stopMitFehler(e);
		} catch (final RuntimeException e) {
			skriptLauf.publiziereErgebnis(e);
			stopMitFehler(e);
		}
	}

	/**
	 * fügt der Liste der nebenlaüfigen Aktivitäten eine neue hinzu.
	 *
	 * @param aktivitaet
	 *            die Aktivität
	 */
	public void addAktivitaet(final UdaAktivitaet aktivitaet) {
		System.err.println(this + " Neue Aktivität: " + aktivitaet);
		if (running) {
			aktivitaeten.put(aktivitaet, null);
		} else {
			aktivitaet.beenden();
		}
	}

	/**
	 * fügt dem Hauptskript des Interpreters eine Anweisung hinzu.
	 *
	 * @param ausdruck
	 *            die neue Anweisung
	 */
	public void addAnweisung(final Ausdruck ausdruck) {
		anweisungsBlock.addAnweisung(ausdruck);
	}

	/**
	 * fügt einen {@link InterpreterListener} hinzu.
	 *
	 * @param listener
	 *            der hinzuzufügende Listener
	 * @see #removeListener(InterpreterListener)
	 */
	public void addListener(final InterpreterListener listener) {
		listenerListe.add(listener);
	}

	/**
	 * hält die Ausführung der Anweisungen des Hauptthreads an.<br>
	 * Eine aktive Anweisung wird nicht unterbrochen, das Skript wartet erst vor
	 * Beginn der Ausführung der nächstfolgenden Anweisung auf die Freigabe.
	 */
	public void anhalten() {
		anweisungsBlock.setRunning(false);
	}

	/**
	 * gibt den übergebenen Text über alle Ausgabekanäle des Skripts aus.
	 *
	 * @param daten
	 *            die auszugebenden Daten
	 */
	public void ausgabe(final ProtokollEintrag daten) {
		skriptLauf.ausgeben(daten);
	}

	/**
	 * beendet die Ausführung des Skripts.
	 */
	public void beenden() {
		running = false;
		anweisungsBlock.beenden();
		if (aktivitaeten != null) {
			for (final UdaAktivitaet aktivitaet : aktivitaeten.keySet()) {
				if (aktivitaet != null) {
					aktivitaet.beenden();
				}
			}
		}
	}

	/**
	 * {@inheritDoc}.<br>
	 * Die Funktion implementiert die entsprechende Funktion der Schnittstelle
	 * {@link ApplicationCloseActionHandler}, um zu verhindern, das das Beenden
	 * der Verbindung, die ein Interpreter verwendet hat das gesamte System
	 * beendet.
	 *
	 * @see de.bsvrz.dav.daf.main.ApplicationCloseActionHandler#close(java.lang.String)
	 */
	@Override
	public void close(final String error) {
		connectionError = error;
		running = false;
		if ((skriptLauf != null) && (skriptLauf instanceof ServerSkriptLauf)) {
			beenden();
			((ServerSkriptLauf) skriptLauf).beenden();
		} else {
			beenden();
		}
	}

	/**
	 * setzt die Ausführung eines zuvor angehaltenen Skripts fort.
	 */
	public void fortsetzen() {
		anweisungsBlock.setRunning(true);
	}

	/**
	 * liefert die Datenverteilerverbindung des Interpreters.
	 *
	 * @return connection die Verbindung
	 */
	public ClientDavConnection getConnection() {
		return connection;
	}

	/**
	 * liefert die nuzterdefinierte Funktion mit dem angegebenen Namen.
	 *
	 * @param name
	 *            der Name der gesuchten Funktion.
	 * @return die Funktionsdefinition
	 * @see #setFunktion(String, FunktionsDefinition)
	 */
	public FunktionsDefinition getFunktion(final String name) {
		return (FunktionsDefinition) funktionen.get(name);
	}

	/**
	 * liefert den nuzterdefinierten Quantor mit dem angegebenen Namen.
	 *
	 * @param name
	 *            der Name des gesuchten Quantors.
	 * @return die Quantordefinition
	 * @see #setQuantor(String, QuantorDefinition)
	 */
	public QuantorDefinition getQuantor(final String name) {
		return (QuantorDefinition) quantoren.get(name);
	}

	/**
	 * liefert den Skriptlauf, für den das Interpreterobjekt angelegt wurde.
	 *
	 * @return den Skriptlauf
	 */
	public SkriptLauf getSkriptLauf() {
		return skriptLauf;
	}

	/**
	 * liefert den Timer des Interpreters.
	 *
	 * @return den Timer
	 */
	public Timer getTimer() {
		return skriptTimer;
	}

	/**
	 * ermittelt, ob der Interpreter läuft.
	 *
	 * @return <code>true</code>, wenn er aktiv ist
	 */
	public boolean isRunning() {
		return running;
	}

	/**
	 * entfernt einen {@link InterpreterListener}.
	 *
	 * @param listener
	 *            der zu entfernende Listener
	 * @see #addListener(InterpreterListener)
	 */
	public void removeListener(final InterpreterListener listener) {
		listenerListe.add(listener);
	}

	/**
	 * führt das zugeordnete Uda-Skript aus.<br>
	 * Die Funktion stellt initialisiert Datenverteilerverbindung mit den
	 * gewünschten Parametern und meldet sich als der im Skriptlauf definierte
	 * Benutzer an. Nach erfolgreicher Anmeldung werden die Anweisungen des
	 * Skriptanweisungsblockes nacheinander ausgeführt. Wenn alle Anweisungen
	 * ausgeführt wurden, blockiert der Hauptthread, bis alle bei der Ausführung
	 * des Skripts gestarteten nebenläufigen Aktivitäten beendet wurden.
	 *
	 * @see java.lang.Thread#run()
	 */
	@Override
	public synchronized void run() {
		Exception fehler = null;
		try {
			ClientDavParameters parameters = null;
			if (initialConnection == null) {
				parameters = new ClientDavParameters();
			} else {
				parameters = initialConnection.getClientDavParameters();
			}
			parameters.setApplicationName("UDA-Interpreter: "
					+ skriptLauf.getName());
			skriptLauf.initialisiereProtokolle();
			connection = new ClientDavConnection(parameters);
			connection.connect();
			connection.setCloseHandler(this);
			connection.login(skriptLauf.getBenutzer(), skriptLauf.getPasswd());

			skriptLauf.initialisiereProtokolle();

		} catch (final MissingParameterException e) {
			fehler = e;
		} catch (final CommunicationError e) {
			fehler = e;
		} catch (final ConnectionException e) {
			fehler = e;
		} catch (final InconsistentLoginException e) {
			fehler = e;
		} catch (final RuntimeException e) {
			fehler = e;
		}

		if (fehler == null) {
			if (!UdaInterpreter.handlerRegistriert) {
				UdaInterpreter.handlerRegistriert = true;
				Operator.registerHandler(new InterpreterHandler());
				Operator.registerHandler(new FuzzyLogikHandler());
				Operator.registerHandler(new ArithmetikHandler());
				Operator.registerHandler(new TrendHandler());
				Operator.registerHandler(new ZeitstempelHandler());
				Operator.registerHandler(new DatenStrukturHandler());
				Operator.registerHandler(new ContainerHandler());
				Operator.registerHandler(new ZeichenkettenHandler());
				Operator.registerHandler(new FehlerHandler());
				Operator.registerHandler(new HystereseHandler());
				Operator.registerHandler(new ThreadHandler());
				Operator.registerHandler(new DavHandler());
				Operator.registerHandler(new UdaFuzzyHandler());
				Operator.registerHandler(new JavaHandler());
				Operator.registerHandler(new KonfigurationsHandler());
				Operator.registerHandler(new AnmeldungHandler());
			}

			kontext.set("global", new SkriptKontext(this));
			kontext.set("konstant", new KonstantSkriptKontext(this));
			kontext.set("parameter", new ParameterSkriptKontext(this));
			kontext.set("konfiguration", new KonfigurationsKontext(this));

			try {
				try {
					running = true;
					anweisungsBlock.interpret(kontext);
				} catch (final UdaFehler e) {
					fehler = e;
					System.out.println("Skriptabbruch: " + e + ": "
							+ e.getSkriptName() + " (" + e.getZeile() + "): "
							+ e.getZusatztext());
					UdaFehler ursache = e.getVorgaenger();
					while (ursache != null) {
						System.out.println(ursache.getSkriptName() + " ("
								+ ursache.getZeile() + "): "
								+ ursache.getZusatztext());
						ursache = ursache.getVorgaenger();
					}

					final Iterator<UdaAktivitaet> iterator = aktivitaeten.keySet().iterator();
					while (iterator.hasNext()) {
						final UdaAktivitaet thread = iterator.next();
						thread.beenden();
					}
				}
				do {
					final Iterator<UdaAktivitaet> iterator = aktivitaeten.keySet().iterator();
					while (iterator.hasNext()) {
						final UdaAktivitaet thread = iterator.next();
						if (!thread.isRunning()) {
							iterator.remove();
						}
					}
					if (aktivitaeten.isEmpty()) {
						break;
					}
					try {
						synchronized (this) {
							wait();
						}
					} catch (final InterruptedException e) {
						e.printStackTrace();
						continue;
					}
				} while (true);

				if (connectionError != null) {
					fehler = new InterpreterException(connectionError);
				}

				skriptTimer.cancel();
				connection.disconnect(false, "Alles OK");

			} catch (final RuntimeException e) {
				fehler = e;
				e.printStackTrace();
				skriptTimer.cancel();
				connection.disconnect(true, e.getMessage());
			}
		} else {
			skriptTimer.cancel();
			if (connection != null) {
				connection.disconnect(true, fehler.getMessage());
			}
		}

		skriptLauf.publiziereErgebnis(fehler);
		skriptLauf.protokolleAbschliessen();
		skriptLauf.setInterpreter(null);

		for (final InterpreterListener listener : listenerListe) {
			listener.beendet(this);
		}

	}

	/**
	 * fügt dem Skript eine nutzerdefinierte Funktion hinzu.<br>
	 * Die nutzerdefinierten Funktionen werden bereits vor der eigentlichen
	 * Ausführung des Skripts ermittelt, eine Funktion mit gleichem Namen
	 * ersetzt immer die vorige Version, d.h. die Änderung einer Funktion zur
	 * Laufzeit ist nicht möglich.
	 *
	 * @param name
	 *            der Name der Funktion
	 * @param definition
	 *            die Funktionsdefinition
	 * @see #getFunktion(String)
	 */
	public void setFunktion(final String name, final FunktionsDefinition definition) {
		funktionen.put(name, definition);
	}

	/**
	 * fügt dem Skript einen nutzerdefinierte Quantor hinzu.<br>
	 * Die nutzerdefinierten Quantoren werden bereits vor der eigentlichen
	 * Ausführung des Skripts ermittelt, ein Quantor mit gleichem Namen ersetzt
	 * immer die vorige Version, d.h. die Änderung eines Quantors zur Laufzeit
	 * ist nicht möglich.
	 *
	 * @param name
	 *            der Name des Quantors
	 * @param definition
	 *            die Quantordefinition
	 * @see #getQuantor(String)
	 */
	public void setQuantor(final String name, final QuantorDefinition definition) {
		quantoren.put(name, definition);
	}

	/**
	 * die Funktion bricht liefert einen bei der Konstruktion oder dem Starten
	 * des Interpreters aufgetretenen Fehler als Exception zurück.
	 *
	 * @param exception
	 *            die aufgetretene Exception bei der Konstruktion des
	 *            Interpreters
	 * @throws UdaInterpreterException
	 *             die verallgemeinerte Fehlermeldung
	 */
	private void stopMitFehler(final Exception exception) throws UdaInterpreterException {
		skriptTimer.cancel();
		if (exception instanceof UdaInterpreterException) {
			throw (UdaInterpreterException) exception;
		}
		throw new UdaInterpreterException(new UdaParserFehler(exception));
	}
}
