/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten;

import java.util.Arrays;

import org.junit.Assert;
import org.junit.Test;

import de.bsvrz.ibv.uda.interpreter.OperandenListe;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArithmetischerFehler;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;

/**
 * Testet die Ausführung der Division innerhalb des Arithmetikhandlers.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ArithmetikHandlerDivision {

	/**
	 * der verwendete Handler.
	 */
	private static final ArithmetikHandler HANDLER = new ArithmetikHandler();

	/**
	 * der Divisionsoperator.
	 */
	private static final Operator OPERATOR = ArithmetikOperators.DIVISION;

	/**
	 * Ausführung der Operation ohne Operanden.
	 */
	@Test(expected = ArgumentFehler.class)
	public void keineOperanden() {
		ArithmetikHandlerDivision.HANDLER.perform(ArithmetikHandlerDivision.OPERATOR, OperandenListe.get());
	}

	/**
	 * Ausführung der Operation mit zu wenigen Operanden.
	 */
	@Test(expected = ArgumentFehler.class)
	public void zuwenigOperanden() {
		ArithmetikHandlerDivision.HANDLER.perform(ArithmetikHandlerDivision.OPERATOR, OperandenListe.get(12));
	}

	/**
	 * Ausführung der Division.
	 */
	@Test
	public void testDivision() {
		Object result = ArithmetikHandlerDivision.HANDLER.perform(ArithmetikHandlerDivision.OPERATOR,
				OperandenListe.get(12, 5));
		Assert.assertEquals("Summe ist nicht korrekt", 2.4, ((Number) result)
				.doubleValue(), 0);
		Assert.assertEquals("Addition liefert keine Fließkommazahl", Double.class,
				result.getClass());

		result = ArithmetikHandlerDivision.HANDLER.perform(ArithmetikHandlerDivision.OPERATOR,
				OperandenListe.get(12, 5.0));
		Assert.assertEquals("Summe ist nicht korrekt", 2.4, ((Number) result)
				.doubleValue(), 0);
		Assert.assertEquals("Addition liefert keine Fließkommazahl", Double.class,
				result.getClass());

		result = ArithmetikHandlerDivision.HANDLER.perform(ArithmetikHandlerDivision.OPERATOR,
				OperandenListe.get(12.0, 5));
		Assert.assertEquals("Summe ist nicht korrekt", 2.4, ((Number) result)
				.doubleValue(), 0);
		Assert.assertEquals("Addition liefert keine Fließkommazahl", Double.class,
				result.getClass());

		result = ArithmetikHandlerDivision.HANDLER.perform(ArithmetikHandlerDivision.OPERATOR,
				OperandenListe.get(12.0, 5.0));
		Assert.assertEquals("Summe ist nicht korrekt", 2.4, ((Number) result)
				.doubleValue(), 0);
		Assert.assertEquals("Addition liefert keine Fließkommazahl", Double.class,
				result.getClass());

	}

	/**
	 * Ausführung der Operation mit zu vielen Argumenten.
	 */
	@Test(expected = ArgumentFehler.class)
	public void zuvieleOperanden() {
		ArithmetikHandlerDivision.HANDLER.perform(ArithmetikHandlerDivision.OPERATOR, OperandenListe.get(12, 13, 14));
	}

	/**
	 * Ausführung der Operation mit Operanden vom falschen Typ.
	 */
	@Test(expected = ArgumentFehler.class)
	public void falscheOperanden() {
		ArithmetikHandlerDivision.HANDLER.perform(ArithmetikHandlerDivision.OPERATOR, OperandenListe.get("Ein Text"));
	}

	/**
	 * ermittelt, ob die erwarteten Operatoren vom Handler unterstützt werden.
	 */
	@Test
	public void testGetHandledOperators() {
		Assert.assertFalse("Anzahl der Operatoren ist 0", 0 == ArithmetikHandlerDivision.HANDLER
				.getHandledOperators().length);
		Assert.assertTrue(Arrays.asList(ArithmetikHandlerDivision.HANDLER.getHandledOperators())
				.contains(ArithmetikHandlerDivision.OPERATOR));
	}

	/**
	 * testet die Funktion validiereHandler für verschiedene
	 * Operandenkombinationen.
	 */
	@Test
	public void testIsHandlerOperatorListOfQextendsObject() {
		Assert.assertFalse(ArithmetikHandlerDivision.HANDLER
				.validiereHandler(ArithmetikHandlerDivision.OPERATOR, OperandenListe.get()).isValid());
		Assert.assertFalse(ArithmetikHandlerDivision.HANDLER
				.validiereHandler(ArithmetikHandlerDivision.OPERATOR, OperandenListe.get(12.0)).isValid());
		Assert.assertFalse(ArithmetikHandlerDivision.HANDLER
				.validiereHandler(ArithmetikHandlerDivision.OPERATOR, OperandenListe.get(12)).isValid());
		Assert.assertFalse(ArithmetikHandlerDivision.HANDLER
				.validiereHandler(ArithmetikHandlerDivision.OPERATOR, OperandenListe.get("Ein Text")).isValid());
		Assert.assertFalse(ArithmetikHandlerDivision.HANDLER
				.validiereHandler(ArithmetikHandlerDivision.OPERATOR, OperandenListe.get(12, 13, 14)).isValid());
		Assert.assertTrue(ArithmetikHandlerDivision.HANDLER
				.validiereHandler(ArithmetikHandlerDivision.OPERATOR, OperandenListe.get(12.0, 5.0)).isValid());
		Assert.assertTrue(ArithmetikHandlerDivision.HANDLER
				.validiereHandler(ArithmetikHandlerDivision.OPERATOR, OperandenListe.get(12.0, 5)).isValid());
		Assert.assertTrue(ArithmetikHandlerDivision.HANDLER
				.validiereHandler(ArithmetikHandlerDivision.OPERATOR, OperandenListe.get(12, 5.0)).isValid());
		Assert.assertTrue(ArithmetikHandlerDivision.HANDLER
				.validiereHandler(ArithmetikHandlerDivision.OPERATOR, OperandenListe.get(12, 5)).isValid());
	}

	/**
	 * Division einer ganzen Zahl durch 0 testen.
	 */
	@Test(expected = ArithmetischerFehler.class)
	public void divisionDurchNullGanz() {
		ArithmetikHandlerDivision.HANDLER.perform(ArithmetikHandlerDivision.OPERATOR, 12, 0);
	}

	/**
	 * Division einer Fliesskommazahl durch 0 testen.
	 */
	@Test(expected = ArithmetischerFehler.class)
	public void divisionDurchNullFliess() {
		ArithmetikHandlerDivision.HANDLER.perform(ArithmetikHandlerDivision.OPERATOR, 12.0, 0.0);
	}
}
