/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.trend;

import org.junit.Assert;
import org.junit.Test;

import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.ibv.uda.interpreter.daten.zeit.Zeitstempel;

/**
 * Testklasse für die die Klasse: {@link TrendEintrag}.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class TrendEintragTest {

	/**
	 * der verwendete Eintrag.
	 */
	private TrendEintrag eintrag;

	/**
	 * Konstruktor mit leeren Argumenten.
	 */
	@Test(expected = ArgumentFehler.class)
	public void konstruktorNullArgument() {
		new TrendEintrag(null, null);
	}

	/**
	 * Auslesen der Zeit aus dem Eintrag.
	 */
	@Test
	public void testGetZeit() {
		final Zeitstempel zs = new Zeitstempel();
		eintrag = new TrendEintrag(zs, 12);
		Assert.assertNotNull(eintrag);
		Assert.assertEquals(zs.getMilliSekunden(), eintrag.getZeit());
	}

	/**
	 * Auslesen des Wertes aus einem Eintrag.
	 */
	@Test
	public void testGetWert() {
		final Zeitstempel zs = new Zeitstempel();
		eintrag = new TrendEintrag(zs, 12);
		Assert.assertNotNull(eintrag);
		Assert.assertEquals(12, eintrag.getWert());
	}

	/**
	 * Test des Operators "fallend".
	 */
	@Test
	public void testFallend() {
		eintrag = new TrendEintrag(new Zeitstempel(), 12);
		Assert.assertFalse("NichtFallend", eintrag.fallend(new TrendEintrag(new Zeitstempel(), 13)).getBoolWert());
		Assert.assertFalse("Gleich", eintrag.fallend(new TrendEintrag(new Zeitstempel(), 12)).getBoolWert());
		Assert.assertTrue("fallend", eintrag.fallend(new TrendEintrag(new Zeitstempel(), 11)).getBoolWert());
	}

	/**
	 * Test des Operators "steigend".
	 */
	@Test
	public void testSteigend() {
		eintrag = new TrendEintrag(new Zeitstempel(), 12);
		Assert.assertFalse(eintrag.steigend(new TrendEintrag(new Zeitstempel(), 11)).getBoolWert());
		Assert.assertFalse(eintrag.steigend(new TrendEintrag(new Zeitstempel(), 12)).getBoolWert());
		Assert.assertTrue(eintrag.steigend(new TrendEintrag(new Zeitstempel(), 13)).getBoolWert());
	}

	/**
	 * Test des Operators "konstant".
	 */
	@Test
	public void testKonstant() {
		eintrag = new TrendEintrag(new Zeitstempel(null), 20L);
		TrendEintrag eintrag2 = new TrendEintrag(new Zeitstempel(null), 20L);
		Assert.assertTrue(eintrag.konstant(eintrag2).getBoolWert());
		Assert.assertTrue(eintrag2.konstant(eintrag).getBoolWert());

		eintrag2 = new TrendEintrag(new Zeitstempel(null), 30L);
		Assert.assertFalse(eintrag.konstant(eintrag2).getBoolWert());
		Assert.assertFalse(eintrag2.konstant(eintrag).getBoolWert());
	}

	/**
	 * Methode für {@link java.lang.Object#toString()} testen.
	 */
	@Test
	public void testToString() {
		eintrag = new TrendEintrag(new Zeitstempel("11.11.2006 12:00"), 20L);
		Assert.assertNotSame("(11.11.2006 12:00:00.000: 20)", eintrag.toString());
	}
}
