/*
 * Segment 6 Intelligente Bewertungsverfahren, SWE 6.3 Umfassende Datenanalyse
 * Copyright (C) 2007-2015 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.ibv.uda.interpreter.daten.zeit;

import java.util.Arrays;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.util.List;

import org.junit.Assert;
import org.junit.Test;

import de.bsvrz.ibv.uda.interpreter.OperandenListe;
import de.bsvrz.ibv.uda.interpreter.daten.fehler.ArgumentFehler;
import de.bsvrz.sys.funclib.bitctrl.interpreter.Operator;

/**
 * Testklasse zum Test der Klasse: {@link ZeitstempelHandler}.
 *
 * @author BitCtrl Systems GmbH, Peuker
 */
public class ZeitstempelHandlerTest {

	/**
	 * der verwendete Handler.
	 */
	private static final ZeitstempelHandler HANDLER = new ZeitstempelHandler();

	/**
	 * der jeweils zu testende Operator.
	 */
	private Operator operator;

	/**
	 * Prüfen der Verfügbarkeit der Operatoren.
	 */
	@Test
	public void testGetHandledOperators() {
		final List<Operator> operatoren = Arrays.asList(ZeitstempelHandlerTest.HANDLER.getHandledOperators());

		Assert.assertTrue(operatoren.size() > 0);
		Assert.assertTrue(operatoren.contains(ZeitstempelOperatoren.FORMAT));
		Assert.assertTrue(operatoren.contains(ZeitstempelOperatoren.KONSTRUKTOR));
		Assert.assertTrue(operatoren.contains(ZeitstempelOperatoren.MINUS));
		Assert.assertTrue(operatoren.contains(ZeitstempelOperatoren.PLUS));
		Assert.assertTrue(operatoren.contains(ZeitstempelOperatoren.ZEITBEREICH_KONSTRUKTOR));
	}

	/**
	 * Test der validiereHandler-Funktion für den Konstruktor.
	 */
	@Test
	public void testIsHandlerKonstruktor() {
		operator = ZeitstempelOperatoren.KONSTRUKTOR;

		Assert.assertTrue(ZeitstempelHandlerTest.HANDLER.validiereHandler(operator, OperandenListe.get()).isValid());
		Assert.assertTrue(
				ZeitstempelHandlerTest.HANDLER.validiereHandler(operator, OperandenListe.get("teststring")).isValid());
		Assert.assertFalse(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get("teststring", "teststring")).isValid());
		Assert.assertFalse(
				ZeitstempelHandlerTest.HANDLER.validiereHandler(operator, OperandenListe.get(new Object())).isValid());
	}

	/**
	 * Test der Validierung der Operation MINUS für den Handler.
	 */
	@Test
	public void testIsHandlerMinus() {
		operator = ZeitstempelOperatoren.MINUS;

		Assert.assertFalse(ZeitstempelHandlerTest.HANDLER.validiereHandler(operator, OperandenListe.get()).isValid());
		Assert.assertFalse(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(new Zeitstempel())).isValid());
		Assert.assertFalse(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(new Zeitstempel(), 12.0)).isValid());
		Assert.assertFalse(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(new Zeitstempel(), 12)).isValid());
		Assert.assertTrue(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(new Zeitstempel(), new Zeitstempel())).isValid());
		Assert.assertTrue(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(new Zeitstempel(), new ZeitDauer(12))).isValid());
		Assert.assertFalse(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(new Zeitstempel(), new ZeitDauer(12), new ZeitDauer(12)))
				.isValid());
		Assert.assertFalse(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(new ZeitDauer(12), new Zeitstempel())).isValid());
		Assert.assertFalse(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(new Object(), new Object())).isValid());
	}

	/**
	 * Test der Validierung der Operation FORMAT für den Handler.
	 */
	@Test
	public void testIsHandlerFormat() {
		operator = ZeitstempelOperatoren.FORMAT;

		Assert.assertFalse(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(new Object(), new Object())).isValid());
		Assert.assertFalse(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(new Zeitstempel(), new Object())).isValid());
		Assert.assertTrue(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(new Zeitstempel(), "")).isValid());
		Assert.assertFalse(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(new Zeitstempel(), "", "")).isValid());
	}

	/**
	 * Test der Validierung der Operation PLUS für den Handler.
	 */
	@Test
	public void testIsHandlerPlus() {
		operator = ZeitstempelOperatoren.PLUS;

		Assert.assertFalse(ZeitstempelHandlerTest.HANDLER.validiereHandler(operator, OperandenListe.get()).isValid());
		Assert.assertFalse(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(new Zeitstempel())).isValid());
		Assert.assertFalse(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(new Zeitstempel(), 12.0)).isValid());
		Assert.assertFalse(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(new Zeitstempel(), new Zeitstempel())).isValid());
		Assert.assertTrue(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(new Zeitstempel(), new ZeitDauer(12))).isValid());
		Assert.assertTrue(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(new ZeitDauer(12), new ZeitDauer(12))).isValid());
		Assert.assertTrue(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(new ZeitDauer(12), new Zeitstempel())).isValid());
		Assert.assertFalse(ZeitstempelHandlerTest.HANDLER
				.validiereHandler(operator, OperandenListe.get(new ZeitDauer(12), new Zeitstempel(), new Object()))
				.isValid());
	}

	/**
	 * Test der Validierung der Ausführung des Konstruktors.
	 */
	@Test
	public void testPerformKonstruktor() {
		operator = ZeitstempelOperatoren.KONSTRUKTOR;
		GregorianCalendar now = new GregorianCalendar();

		Zeitstempel zs = (Zeitstempel) ZeitstempelHandlerTest.HANDLER.perform(operator, OperandenListe.get());
		Assert.assertTrue((zs.getMilliSekunden() - now.getTimeInMillis()) < 1000L);

		now = new GregorianCalendar(2006, Calendar.DECEMBER, 24);
		zs = (Zeitstempel) ZeitstempelHandlerTest.HANDLER.perform(operator, OperandenListe.get("24.12.2006"));
		Assert.assertTrue((zs.getMilliSekunden() - now.getTimeInMillis()) < 1000L);
	}

	/**
	 * Test der Ausführung des Operators MINUS.
	 */
	@Test
	public void testPerformMinus() {
		operator = ZeitstempelOperatoren.MINUS;

		final Zeitstempel zs1 = new Zeitstempel(10000L + 3600L);
		final Zeitstempel zs2 = new Zeitstempel(10000L);
		final ZeitDauer zd1 = new ZeitDauer(10000L + 3600L);
		final ZeitDauer zd2 = new ZeitDauer(10000L);

		Assert.assertEquals(3600L,
				((ZeitDauer) ZeitstempelHandlerTest.HANDLER.perform(operator, OperandenListe.get(zs1, zs2)))
				.getMilliSekunden());

		Assert.assertEquals(-3600L,
				((ZeitDauer) ZeitstempelHandlerTest.HANDLER.perform(operator, OperandenListe.get(zs2, zs1)))
				.getMilliSekunden());

		Assert.assertEquals(0L,
				((Zeitstempel) ZeitstempelHandlerTest.HANDLER.perform(operator, OperandenListe.get(zs1, zd1)))
				.getMilliSekunden());

		try {
			Assert.assertEquals(0L, ZeitstempelHandlerTest.HANDLER.perform(operator, OperandenListe.get(zd1, zs1)));
			Assert.fail("Von einer Zeitdauer kann kein Zeitstempel abgezogen werden");
		} catch (final ArgumentFehler e) {
			// Ungültige Argumente erkannt
		}

		Assert.assertEquals(3600L,
				((ZeitDauer) ZeitstempelHandlerTest.HANDLER.perform(operator, OperandenListe.get(zd1, zd2)))
				.getMilliSekunden());

		Assert.assertEquals(-3600L,
				((ZeitDauer) ZeitstempelHandlerTest.HANDLER.perform(operator, OperandenListe.get(zd2, zd1)))
				.getMilliSekunden());
	}

	/**
	 * Test der Ausführung des Operators PLUS.
	 */
	@Test
	public void testPerformPlus() {
		operator = ZeitstempelOperatoren.PLUS;

		final Zeitstempel zs1 = new Zeitstempel(10000L + 3600L);
		final Zeitstempel zs2 = new Zeitstempel(10000L);
		final ZeitDauer zd1 = new ZeitDauer(10000L + 3600L);
		final ZeitDauer zd2 = new ZeitDauer(10000L);

		try {
			Assert.assertEquals(23600L, ZeitstempelHandlerTest.HANDLER.perform(operator, OperandenListe.get(zs1, zs2)));
			Assert.fail("Zeitstempel können nicht addiert werden");
		} catch (final ArgumentFehler e) {
			// Zeitstempel können nicht addiert werden
		}

		try {
			Assert.assertEquals(23600L, ZeitstempelHandlerTest.HANDLER.perform(operator, OperandenListe.get(zs2, zs1)));
			Assert.fail("Zeitstempel können nicht addiert werden");
		} catch (final ArgumentFehler e) {
			// Zeitstempel können nicht addiert werden
		}

		Assert.assertEquals(27200L,
				((Zeitstempel) ZeitstempelHandlerTest.HANDLER.perform(operator, OperandenListe.get(zs1, zd1)))
				.getMilliSekunden());

		Assert.assertEquals(27200L,
				((Zeitstempel) ZeitstempelHandlerTest.HANDLER.perform(operator, OperandenListe.get(zd1, zs1)))
				.getMilliSekunden());

		Assert.assertEquals(23600L,
				((ZeitDauer) ZeitstempelHandlerTest.HANDLER.perform(operator, OperandenListe.get(zd1, zd2)))
				.getMilliSekunden());

		Assert.assertEquals(23600L,
				((ZeitDauer) ZeitstempelHandlerTest.HANDLER.perform(operator, OperandenListe.get(zd2, zd1)))
				.getMilliSekunden());
	}

	/**
	 * Test der Ausführung des Operators FORMAT.
	 */
	@Test
	public void testPerformFormat() {
		operator = ZeitstempelOperatoren.FORMAT;
		final Zeitstempel zs = new Zeitstempel("24.12.2006");

		Assert.assertEquals("24.12.06 00:00:00", ZeitstempelHandlerTest.HANDLER.perform(operator,
				OperandenListe.get(zs, "TT.MM.JJ HH:mm:ss")));
		Assert.assertEquals("24.12.2006 00:00:00,000", ZeitstempelHandlerTest.HANDLER.perform(operator,
				OperandenListe.get(zs)));
	}
}
